@@ -2,9 +2,9 @@ package URI;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = "1.30"; # $Date: 2004/01/14 13:59:48 $
+$VERSION = "1.37";
 
-use vars qw($ABS_REMOTE_LEADING_DOTS $ABS_ALLOW_RELATIVE_SCHEME);
+use vars qw($ABS_REMOTE_LEADING_DOTS $ABS_ALLOW_RELATIVE_SCHEME $DEFAULT_QUERY_FORM_DELIMITER);
 
 my %implements;  # mapping from scheme to implementor class
 
@@ -73,7 +73,8 @@ sub _init
 {
     my $class = shift;
     my($str, $scheme) = @_;
-    $str =~ s/([^$uric\#])/$URI::Escape::escapes{$1}/go;
+    # find all funny characters and encode the bytes.
+    $str =~ s*([^$uric\#])* URI::Escape::escape_char($1) *ego;
     $str = "$scheme:$str" unless $str =~ /^$scheme_re:/o ||
                                  $class->_no_scheme_ok;
     my $self = bless \$str, $class;
@@ -204,7 +205,7 @@ sub opaque
 
     my $new_opaque = shift;
     $new_opaque = "" unless defined $new_opaque;
-    $new_opaque =~ s/([^$uric])/$URI::Escape::escapes{$1}/go;
+    $new_opaque =~ s/([^$uric])/ URI::Escape::escape_char($1)/ego;
 
     $$self = defined($old_scheme) ? $old_scheme : "";
     $$self .= $new_opaque;
@@ -229,7 +230,7 @@ sub fragment
 
     my $new_frag = shift;
     if (defined $new_frag) {
-	$new_frag =~ s/([^$uric])/$URI::Escape::escapes{$1}/go;
+	$new_frag =~ s/([^$uric])/ URI::Escape::escape_char($1) /ego;
 	$$self .= "#$new_frag";
     }
     $old;
@@ -245,20 +246,26 @@ sub as_string
 
 sub canonical
 {
-    my $self = shift;
+    # Make sure scheme is lowercased, that we don't escape unreserved chars,
+    # and that we use upcase escape sequences.
 
-    # Make sure scheme is lowercased
+    my $self = shift;
     my $scheme = $self->_scheme || "";
     my $uc_scheme = $scheme =~ /[A-Z]/;
-    my $lc_esc    = $$self =~ /%(?:[a-f][a-fA-F0-9]|[A-F0-9][a-f])/;
-    if ($uc_scheme || $lc_esc) {
-	my $other = $self->clone;
-	$other->_scheme(lc $scheme) if $uc_scheme;
-	$$other =~ s/(%(?:[a-f][a-fA-F0-9]|[A-F0-9][a-f]))/uc($1)/ge
-	    if $lc_esc;
-	return $other;
+    my $esc = $$self =~ /%[a-fA-F0-9]{2}/;
+    return $self unless $uc_scheme || $esc;
+
+    my $other = $self->clone;
+    if ($uc_scheme) {
+	$other->_scheme(lc $scheme);
     }
-    $self;
+    if ($esc) {
+	$$other =~ s{%([0-9a-fA-F]{2})}
+	            { my $a = chr(hex($1));
+                      $a =~ /^[$unreserved]\z/o ? $a : "%\U$1"
+                    }ge;
+    }
+    return $other;
 }
 
 # Compare two URIs, subclasses will provide a more correct implementation
@@ -564,6 +571,10 @@ parameter strings.  Such an anonymous array uses overloading so it can
 be treated as a string too, but this string does not include the
 parameters.
 
+Note that absolute paths have the empty string as their first
+I<path_segment>, i.e. the I<path> C</foo/bar> have 3
+I<path_segments>; "", "foo" and "bar".
+
 =item $uri->query
 
 =item $uri->query( $new_query )
@@ -575,10 +586,16 @@ the $uri.
 
 =item $uri->query_form( $key1 => $val1, $key2 => $val2, ... )
 
+=item $uri->query_form( $key1 => $val1, $key2 => $val2, ..., $delim )
+
 =item $uri->query_form( \@key_value_pairs )
 
+=item $uri->query_form( \@key_value_pairs, $delim )
+
 =item $uri->query_form( \%hash )
 
+=item $uri->query_form( \%hash, $delim )
+
 Sets and returns query components that use the
 I<application/x-www-form-urlencoded> format.  Key/value pairs are
 separated by "&", and the key is separated from the value by a "="
@@ -603,6 +620,13 @@ All the following statements have the same effect:
     $uri->query_form([ foo => [1, 2] ]);
     $uri->query_form({ foo => [1, 2] });
 
+The $delim parameter can be passed as ";" to force the key/value pairs
+to be delimited by ";" instead of "&" in the query string.  This
+practice is often recommened for URLs embedded in HTML or XML
+documents as this avoids the trouble of escaping the "&" character.
+You might also set the $URI::DEFAULT_QUERY_FORM_DELIMITER variable to
+";" for the same global effect.
+
 The C<URI::QueryParam> module can be loaded to add further methods to
 manipulate the form of a URI.  See L<URI::QueryParam> for details.
 
@@ -897,7 +921,8 @@ The C<urn:isbn:> namespace contains International Standard Book
 Numbers (ISBNs) and is described in RFC 3187.  A C<URI> object belonging
 to this namespace has the following extra methods (if the
 Business::ISBN module is available): $uri->isbn,
-$uri->isbn_publisher_code, $uri->isbn_country_code, $uri->isbn_as_ean.
+$uri->isbn_publisher_code, $uri->isbn_group_code (formerly isbn_country_code,
+which is still supported by issues a deprecation warning), $uri->isbn_as_ean.
 
 =item B<urn>:B<oid>:
 
@@ -946,6 +971,11 @@ examples:
   URI->new("../../../foo")->abs("http://host/a/b")
       ==> "http://host/foo"
 
+=item $URI::DEFAULT_QUERY_FORM_DELIMITER
+
+This value can be set to ";" to have the query form C<key=value> pairs
+delimited by ";" instead of "&" which is the default.
+
 =back
 
 =head1 BUGS
@@ -985,7 +1015,7 @@ http://www.w3.org/Addressing/
 
 =head1 COPYRIGHT
 
-Copyright 1995-2003 Gisle Aas.
+Copyright 1995-2004,2008 Gisle Aas.
 
 Copyright 1995 Martijn Koster.
 
